;********************************************
;* Programy asemblerowe     14-05-2016		*
;********************************************
			AREA AsmQ32, CODE, READONLY

;****** funkcje uniwersalne Q32 - nie zaleza od formatu *******
;------------------ Q32 wartosc bezwzgledna -------------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem
;OUTPUT: r0 = jezeli ujemna -> zmieniec znak na +
			EXPORT Q32_abs
			ALIGN
Q32_abs		PROC
			MOVS	r1,#1		; maska:
			ROR		r1,#1		; r1 = 0x80000000
			TST		r0,r1		; bit r0[31]
			BNE		Q32_neg		; skocz, liczba ujemna
			BX		lr
			ENDP
;------------------------ Q32 negacja -------------------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem
;OUTPUT: r0 = zamienia znak na przeciwny (z nasyceniem)
			EXPORT Q32_neg
			ALIGN
Q32_neg		PROC
			MOVS	r2,r0		; pamietaj znak oryginalu
			MVNS	r0,r0		; negacja bit po bicie
			ADDS 	r0,#1		; r0 = r0 + 1, Flagi
			BVC  	label01		; skocz, gdy overflow clear
			MOVS 	r0,#1
			ROR	 	r0,#1		; w r0 mam 0x80000000
			TST		r2,r0
			BEQ  	label01		; pamietany znak jest odwrotny
			MVNS 	r0,r0		; negacja bit po bicie
label01		BX   	lr
			ENDP				

;---------------- Q32 odejmowanie z nasyceniem ----------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem
;		 r1 = liczba stalopozycyjna ze znakiem
;OUTPUT: r0 = r0 - r1
			EXPORT Q32_sub
			ALIGN
Q32_sub		PROC
			SUBS 	r0,r1		; r0 = r0 - r1, Flagi
			BVS  	label02		; skocz, gdy overflow set
			BX   	lr
			ENDP
;----------------- Q32 dodawanie z nasyceniem -----------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem
;		 r1 = liczba stalopozycyjna ze znakiem
;OUTPUT: r0 = r0 + r1
			EXPORT Q32_add
			ALIGN
Q32_add		PROC
			ADDS	r0,r1		; r0 = r0 + r1, Flagi
			BVC		label03		; skocz, gdy overflow clear
label02		MOVS	r1,r0
			MOVS	r0,#1
			ROR		r0,#1		; w r0 mam 0x80000000
			TST		r1,r0
			BEQ  	label03		; pamietany znak jest odwrotny
			MVNS 	r0,r0		; negacja bit po bicie
label03		BX   	lr
			ENDP

; ************** Q15.16 funcje zalezne od formatu *************
;------------ zamiana reprezentacji Q15.16 na int -------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem Q15.16
;OUTPUT: r0 = liczba calkowita int (4 bajty)	 
			EXPORT Q1516_int
			ALIGN
Q1516_int	PROC
			ASR		r0,#16
			BX   	lr
			ENDP
;----------- zamiana reprezentacji Q15.16 na float ------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem Q15.16
;OUTPUT: r0 = liczba zmiennopozycyjna float (4 bajty)	 
			EXPORT Q1516_float
			ALIGN
Q1516_float	PROC
			MOVS 	r1,r0		; ustaw flagi
			BEQ	 	label07		; gdy zero, to koniec
			LSLS 	r0,r0,#1	; bit znaku do Carry
			MOVS  	r0,#0		; wynik bedzie w r0
			BCC  	label04		; skocz, gdy C = 0
			MVNS	r1,r1		; negacja bit po bicie
			ADD	 	r1,#1		; w r1 jest liczba nieujemna
			ORR	 	r0,r0,#(1<<31)	; bit znaku jest jedynka
label04		MOVS 	r2,#32		; licznik -> wykladnik
label05		LSLS 	r1,r1,#1	; bit znaku do Carry
			BCS  	label06		; skocz, gdy C = 1 (mamy jedynke)
			SUBS 	r2,#1		; licznik --
			BNE	 	label05		; jeszcze nie koniec
label06		UBFX 	r3,r1,#9,#23	; mantysa (bez jedynki) do r3
			ORR	 	r0,r0,r3
			ADD	 	r2,#110		; przesuniecie wykladnika
			LSL	 	r2,r2,#23
			ORR	 	r0,r0,r2
label07		BX   	lr
			ENDP
;------------------ zamiana float na Q15.16 -------------------
;INPUT:  r0 = liczba zmiennopozycyjna float (4 bajty)	 
;OUTPUT: r0 = liczba stalopozycyjna ze znakiem Q15.16
			EXPORT float_Q1516
			ALIGN
float_Q1516	PROC
			PUSH	{r4-r8,lr}	; parzysta ilosc rejestrow
			MOVS 	r1,r0		; ustaw flagi
			BEQ	 	label14		; gdy zero, to koniec
			; maski:
			MOVS	r6,#0
			MVNS	r7,r6
			LSR		r7,#1		; w r7 mam	0x7FFFFFFF
			MVNS	r6,r7		; w r6 mam	0x80000000
			; czy minus zero?
			EORS	r0,r6
			BEQ		label10		; koniec gdy bylo minus zero
			; czy przekroczenie zakresu
			TST		r1,r6		; jaki znak liczby?
			BNE		label08		; skocz, gdy (-)
			LDR		r4,=0x46FFFFFF	; float rowny 0x7FFFFFFF
			CMP		r4,r1		; r4 - r1
			BGT		label09		; skocz gdy r4 > r1
			MOVS	r0,r7		; r0 = 0x7FFFFFFFF
			B		label14
label08		MOVS	r0,r6		; r0 = 0x80000000
			LDR		r4,=0xC7000000	; float rowny 0x80000000
			CMP		r4,r1		; r4 - r1
			BLT		label14		; skocz gdy r4 < r1
			; rozpakujemy liczbe
label09		UBFX	r2,r1,#23,#8	; r2 = exponent
			EORS	r2,#0xFF	; czy NAN
			BNE		label11		; skocz gdy nie
			MOVS	r0,r7		; r0 = 0x7FFFFFFF
			TST		r1,r6
			BEQ		label10
			MOVS	r0,r6		; r0 = 0x80000000
label10		POP		{r4-r8,pc}
; TU: poprawna liczba
label11		UBFX	r2,r1,#23,#8	; r2 = exponent
			SUBS	r2,#127		; r2 = true exponent	
			UBFX	r3,r1,#0,#23	; r3 = fraction (bez jedynki)
			LSR		r5,r6,#8
			ORR		r3,r3,r5	; dopisz jedynke przed kropka
			SUBS	r4,r2,#7
			BMI		label12		; skocz gdy minus
			LSL		r0,r3,r4	;TU: plus <-- w lewo
			B		label13
label12		MVNS	r4,r4		;TU: minus --> w prawo
			ADDS	r4,#1
			LSR		r0,r3,r4
			; znak wyniku
label13		TST		r1,r6		; test znaku
			BEQ		label14
			MVNS	r0,r0
			ADDS	r0,#1		; negacja liczby
label14		POP		{r4-r8,pc}
			ENDP
;---------------- Q15.16 mnozenie z nasyceniem ----------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem Q15.16
;		 r1 = liczba stalopozycyjna ze znakiem Q15.16
;OUTPUT: r0 = r0 * r1
			EXPORT Q1516_mult
			ALIGN
Q1516_mult	PROC
			PUSH	{r4-r8,lr}	; parzysta ilosc rejestrow
			; maski:
			MOVS	r4,#0
			MVNS	r5,r4
			LSL		r5,#16		; w r5 mam 	0xFFFF0000
			MVNS	r6,r4
			LSR		r6,#1		; w r6 mam	0x7FFFFFFF
			MVNS	r4,r6		; w r4 mam	0x80000000
			; signed long multiply:
			SMULL	r2,r3,r0,r1	; (r3:r2) = r0 * r1
			; czy wynik jest zerem
			ORRS	r8,r2,r3
			BNE		label15		; skocz, gdy nie
			MOVS	r0,#0
			POP		{r4-r8,pc}
label15		EOR		r7,r0,r1	; r7[b31] to znak wyniku
			MOVS	r1,r3		; do sprawdzenia OVER
			LSLS	r3,r3,#16	; shift left high word
			LSRS	r2,r2,#16	; shift right low word
			ORR		r0,r2,r3	; w r0 jest wynik
			; sprawdzenie OVER
			LSLS  	r7,r7,#1	; bit znaku do Carry
			BCC   	label16		; skocz, gdy C = 0
			ANDS	r8,r0,r4	;TU: wynik musi byc ujemny
			BEQ		label18		; skocz, gdy plus = nasycenie
			MVNS	r2,r1		; najstarsze 16 bitow
			ANDS	r2,r5
			BNE		label18		; skocz, gdy bylo zero = nasycenie
			POP		{r4-r8,pc}			
label16		ANDS	r8,r0,r4	;TU: wynik musi byc dodatni
			BNE		label17		; skocz, gdy minus = nasycenie
			ANDS	r1,r5
			BEQ		label19		; skocz gdy OK
label17		MOVS	r0,r6		; nasycenie (+) r0 = 0x7FFFFFFF
			POP		{r4-r8,pc}
label18		MOVS	r0,r4		; nasycenie (-) r0 = 0x80000000	
label19		POP		{r4-r8,pc}
			ENDP
;---------------- Q15.16 dzielenie z nasyceniem ---------------
;INPUT:  r0 = liczba stalopozycyjna ze znakiem Q15.16
;		 r1 = liczba stalopozycyjna ze znakiem Q15.16
;OUTPUT: r0 = r0 / r1
;dzielenie przez zero powoduje nasycenie zalezne od znaku licznika
			EXPORT Q1516_div
			ALIGN
Q1516_div	PROC
			PUSH	{r4-r8,lr}	; parzysta ilosc rejestrow
			; czy licznik ulamka jest zerem ?
			ORRS	r8,r0,r0
			BNE		label20
			POP		{r4-r8,pc}
			; maski:
label20		MOVS	r2,#0
			MVNS	r3,r2		; w r3 mam 	0xFFFFFFF
			MOVS	r5,r3
			MOVS	r6,r3
			LSL		r5,#16		; w r5 mam 	0xFFFF0000
			LSR		r6,#1		; w r6 mam	0x7FFFFFFF
			MVNS	r4,r6		; w r4 mam	0x80000000
			; czy dzielenie przez zero ?
			EOR		r7,r0,r1	; r7[b31] to znak wyniku
			ANDS	r8,r1,r3	; czy dzielnik jest zerem
			BNE		label21		; skocz, gdy nie
			LSLS  	r0,r0,#1	; bit znaku do Carry
			BCC   	label24		; skocz, gdy C = 0
			B		label25
			; dzielenie ze znakiem:
label21		MOVS	r2,r1		; mianownik do r2
			ASRS	r1,r0,#31	; r1[31:0] bity znaku A
			LSLS	r1,r1,#16	; przesuwamy
			ORR		r1,r1,r0,LSR #16
			LSLS	r0,r0,#16	; w r1:r0 jest rozszerzenie A
			ASRS	r3,r2,#31	; w r3:r2 rozszerzone B			
			BL		Int64_sdiv	; dzielenie liczb calkowitych, wynik w r0
			; sprawdze przekroczenie zakresu
			MOVS	r2,#0
			MVNS	r2,r2		; r2 = 0xFFFFFFF
			ANDS	r3,r1,r2	; czy w r1 sa jedynki ?
			BNE		label22		; skocz, gdy TAK
			ANDS	r3,r0,r4	;TU: r1=0,  bit znaku
			BEQ		label26		; skocz, gdy OK
			B		label24		; over 0x7FFFFFFF
			;TU: w r1 sa jedynki
label22		EORS	r3,r1,r2	; czy w r1 same jedynki
			BNE		label23		; skocz, gdy NIE
			ANDS	r3,r0,r4	;TU: TAK, bit znaku r0 ?
			BNE		label26
			B		label25
			;TU: w r1 sa jedynki i zera
label23		ANDS	r3,r1,r4	; bit znaku
			BEQ		label24
			B		label25
label24		MOVS	r0,r6		; nasycenie (+) r0 = 0x7FFFFFFF
			POP		{r4-r8,pc}
label25		MOVS	r0,r4		; nasycenie (-) r0 = 0x80000000	
label26		POP		{r4-r8,pc}
			ENDP

; ******** INTEGER64 funkcje calkowitoliczbowe, 64-bity *******
;------------- Int64_sdiv: 64-bit Signed Division -------------
;INPUT:  (r1:r0)/(r3:r2)dzielenie liczb calkowitych ze znakiem
;OUTPUT: (r1:r0) czesc calkowita wyniku
;		 (r3:r2) reszta z dzielenia
;			EXPORT Int64_sdiv
			ALIGN
Int64_sdiv	PROC
			PUSH	{r4-r8,lr}	; parzysta ilosc rejestrow
			; maski:
			MOVS	r6,#1
			ROR		r6,#1		; r6 = 0x80000000
			ROR		r7,r6,#1	; r7 = 0x40000000
			; znak wyniku i znak reszty
			ASRS	r4,r1,#1	; gdy r1 < 0, to r4[31:30] = 11,
			EOR		r4,r4,r3,LSR #1	; gdy r3 >= 0 to r4[31:30] = 00
			; gdy r1 >= 0 i r3 >= 0, to r4[31:30] = (00)^(00) = 00
			; gdy r1 >= 0 i r3 <  0, to r4[31:30] = (00)^(01) = 01
			; gdy r1 <  0 i r3 >= 0, to r4[31:30] = (11)^(00) = 11
			; gdy r1 <  0 i r3 <  0, to r4[31:30] = (11)^(01) = 10
			; bit r4[31] oznacza znak dzielnej (licznika)
			; bit r4[30] czy obie liczby sa jednoczesnie dodatnie, czy ujemne
			
			; zamien licznik (r1:r0) na liczbe dodatnia (gdy jest ujemny)
			BPL		label27		; skocz, gdy licznik >= 0
			MVNS	r0,r0		; negacja bit po bicie
			MVNS	r1,r1		; negacja bit po bicie
			ADDS	r0,#1		; negacja U2, mlodszy bajt
			ADC		r1,#0		; (r1:r0) = -(r1:r0)
			; zamien mianownik (r3:r2) na liczbe dodatnia (gdy jest ujemny)
label27		TST		r3,r3		; czy mianownik >= 0
			BPL		label28
			MVNS	r2,r2		; negacja bit po bicie
			MVNS	r3,r3		; negacja bit po bicie
			ADDS	r2,#1		; negacja U2, mlodszy bajt
			ADC		r3,#0		; (r3:r2) = -(r3:r2)
			; teraz dzielenie liczb dodatnich (bez znaku) (r1:r0)/(r3:r2)
label28		BL		Int64_udiv
			; jezeli obie liczby maja rozne znaki, wynik jest ujemny
			TST		r4,r7		; bit r4[30]
			BEQ		label29		; skocz, znaki zgodne
			MVNS	r0,r0
			MVNS	r1,r1
			ADDS	r0,#1
			ADC		r1,#0		; (r1:r0) = -(r1:r0)
			; znak reszty = znak licznika
label29		TST		r4,r6		; bit r4[31]
			BEQ		label30		; skocz, licznik dodatni
			MVNS	r2,r2
			MVNS	r3,r3
			ADDS	r2,#1
			ADC		r3,#0		; (r3:r2) = -(r3:r2)
			; koniec
label30		POP		{r4-r8,pc}
			ENDP
;------------------------- Int64_udiv0 ------------------------
;INPUT:  (r1:r0)/(r3:r2)dzielenie liczb calkowitych dodatnich
;OUTPUT: (r1:r0) czesc calkowita wyniku
;		 (r3:r2) reszta z dzielenia
;			EXPORT Int64_udiv0
			ALIGN
Int64_udiv0	PROC
			PUSH	{r4-r8,lr}	; parzysta ilosc rejestrow
			; maska:
			MOVS	r6,#1
			ROR		r6,#1		; r6 = 0x80000000
			; wartosci poczatkowe rejestrow
			MOVS	r5,r3
			MOVS	r4,r2		; (r5:r4) mianownik
			MOVS	r3,r1
			MOVS	r2,r0		; (r3:r2) reszta
			MOVS	r1,#0
			MOVS	r0,r1		; (r1:r0) wynik
			; reszta = reszta - mianownik (r3:r2) = (r3:r2) - (r5:r4)
label31		SUBS	r2,r2,r4
			SBC		r3,r3,r5
			ANDS	r8,r3,r6	; jaki jest bit znaku
			BNE		label32		; skocz, gdy liczba ujemna
			; wynik++
			ADDS	r0,#1
			ADC		r1,#0
			B		label31
label32		ADDS	r2,r2,r4
			ADC		r3,r3,r5
			POP		{r4-r8,pc}
			ENDP			
;------------------------- Int64_sub --------------------------
;INPUT:  (r1:r0), (r3:r2) dwie liczby
;OUTPUT: (r1:r0) = (r1:r0) - (r3:r2)
;			EXPORT Int64_sub
			ALIGN
Int64_sub	PROC
			PUSH	{r4-r6,lr}	; parzysta ilosc rejestrow
			SUBS	r4,r0,r2	; subtract two lower words (update Carry)
			SBC		r5,r1,r3	; subtract two upper words and Borrow
			MOVS	r1,r5
			MOVS	r0,r4
			POP		{r4-r6,pc}
			ENDP			
;---------------- Int64 Counting Leading Zeros ----------------
;INPUT:  (r1:r0) = (upper:lower)
;OUTPUT: r2 = # of leading zero bits in the 64-bit data
;			EXPORT Int64_LeadingZeros
			ALIGN
Int64_LeadingZeros	PROC
			; Counting # of leading zeroes in upper word
			CLZ		r2,r1		; CLZ = Count Leading Zeros
			; Counting # of leading zeroes in lower word
			CMP		r2,#32
			CLZEQ	r3,r0		; if r2 == 32, then count leading zero
								; bits of the lower word
			ADDEQ	r2,r2,r3	; if all bits of the upper word are zero,
								; add the leading zeroes of the lower word
			BX   	lr
			ENDP			
;------------------- Int64 Logic Shift Left -------------------
;INPUT:  (r1:r0) = (upper:lower)
;		  r2 = przesuniecie
;OUTPUT: (r1:r0) = (upper:lower) przesuniete
;			EXPORT Int64_ShiftLeft
			ALIGN
Int64_ShiftLeft	PROC
			PUSH	{r4-r6,lr}
			MOV		r3,r0		; backup the lower word
			MOV		r1,r1, LSL r2	; shift left the upper word
			MOV		r0,r0, LSL r2	; shift left the lower word
			; shift bits of the lower word into the upper word
			CMP		r2,#32
			; if r2 < 32
			RSBLO	r5,r2,#32	; r5 = 32 - r2
			LSR		r4,r3,r5	; r4 = r3 >> (32 - r2)
			ORRLO	r1,r1,r4	; upper |= lower >> (32 - r2)
			; if r2 > 32
			SUBHS	r5,r2,#32	; r5 = r2 - 32
			LSLHS	r1,r3,r5	; upper = lower << (r2 - 32)
			POP		{r4-r6,pc}
			ENDP	

;------------------------- Int64_udiv -------------------------
;INPUT:  (r1:r0)/(r3:r2)dzielenie liczb calkowitych dodatnich
;OUTPUT: (r1:r0) czesc calkowita wyniku
;		 (r3:r2) reszta z dzielenia
;			EXPORT Int64_udiv
			ALIGN
Int64_udiv	PROC
			PUSH	{r4-r12,lr}	; parzysta ilosc rejestrow
			MOVS	r8,#0		; (r1:r0) reminder
			MOVS	r9,r8		; (r9:r8) quotient = 0
label33		PUSH	{r0-r3}
			POP		{r4-r7}
			; pomin, gdy licznik < mianownik
			BL		Int64_sub	; (r1:r0) = (r1:r0) - (r3:r2)
			MOVS	r10,r1
			PUSH	{r4-r7}
			POP		{r0-r3}
			TST		r10,#0x80000000	; check the sign bit
			BNE		label36
			; (1) a_64_bit
			PUSH	{r0-r3}
			BL		Int64_LeadingZeros	; INPUT:  (r1:r0), r2 = ilosc zer
			MOVS	r10,r2		; r10 = x
			BL		Int64_ShiftLeft
			MOVS	r4,r0
			MOVS	r5,r1
			; (2) b_64_bit
			MOVS	r0,r6
			MOVS	r1,r7
			BL		Int64_LeadingZeros	; INPUT:  (r1:r0), r2 = ilosc zer
			MOVS	r11,r2			; r11 = y
			BL		Int64_ShiftLeft
			MOVS	r6,r0
			MOVS	r7,r1
			; (3) g_32_bit
label34		UDIV	r12,r5,r7	; r12 = g_32_bit
			ORRS	r12,#0
			BNE		label35
			; za duzy podzielnik, przesun (r7:r6) w prawo
			SUB		r11,#1
			LSRS	r7,#1
			RRX		r6,r6
			B		label34
label35		MOVS	r0,r12
			MOVS	r1,#0
			SUB		r2,r11,r10
			BL		Int64_ShiftLeft
			; quotient = quotient + wynik dzielenia
			ADDS	r8,r8,r0	; mlodsze
			ADC		r9,r9,r1	; starsze
			; remainder = remainder - wynik * divisor
			MOVS	r4,r0
			MOVS	r5,r1		; (r5:r4) = wynik dzielenia
			POP		{r0-r3}
			; mnozenie (wynik)*(divisor)
			UMULL	r6,r7,r4,r2	; (r7:r6) = r4 * r2
			MLA		r7,r5,r2,r7	; r7 = r7 + r5 * r2
			MLA		r7,r4,r3,r7	; r7 = r7 + r4 * r3
			; remainder = remainder - (wynik)*(divisor)
			PUSH	{r2-r3}
			MOVS	r3,r7
			MOVS	r2,r6
			BL		Int64_sub	; (r1:r0) = (r1:r0) - (r3:r2)
			POP		{r2-r3}
			B		label33
			; koniec
			; copy the results to the return registers
label36		MOVS	r3,r1
			MOVS	r2,r0		; reminder
			MOVS	r1,r9
			MOVS	r0,r8		; quotient
			POP		{r4-r12,pc}
			ENDP

;----------------------------------------------------------------
			EXPORT Int64_test
			ALIGN
Int64_test	PROC
			PUSH	{r4,lr}
			LDR		r1,=0x00000000
			LDR		r0,=0x000000E0
			LDR		r3,=0x00000000
			LDR		r2,=0x00000005
			BL		Int64_udiv
			POP		{r4,pc}
			ENDP	


			END
				

/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



