/******************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 	*
 *  STM3210E-EVAL:  PROJEKT_03 bit parzystosci 				*
 ******************************************************/
#include "stm32f10x.h"
#include "stm3210e_eval_lcd.h"
#include <stdbool.h> 		// true, false
#include "AsmCode.h"
#define Brown 0xEBC2
#define Amber 0xFDEA

int  RCC_Config(void);
void GPIO_Config(void);
void Delay(unsigned int);

int main(void) {
  unsigned int k, i;
	uint8_t str2disp[19] = {"                  \0"};
	bool parity;
	
  //konfiguracja systemu --------------------------------------
  RCC_Config();		
  GPIO_Config(); 
  GPIO_ResetBits(GPIOF, GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9);	// zgasimy wszystkie diody LED
  STM3210E_LCD_Init();		// inicjalizacja TFT LCD 240320L8
	LCD_Clear(Black);		// Clear the LCD
	
	// napis ----------------------------------------------------
  LCD_SetBackColor(Black);		// kolor tla
  LCD_SetTextColor(Amber);		// kolor textu
	LCD_SetFont(&Font16x24);
  LCD_DisplayStringLine(Line0, (uint8_t *)"    Projekt03_asm   ");
	LCD_DisplayStringLine(Line2, (uint8_t *)"bit parzystosci:    ");
	LCD_DisplayStringLine(Line4, (uint8_t *) str2disp);
	Delay(8000000);
	
// ---------------------- Bit parzystosci ---------------------
	for(k=0; k < 256; k++){
		// wyswietl 'k' jako 8 bitow
		for(i = 0; i < 8; i++) {
			str2disp[12-i] = (k & (1 << i))?'1':'0';
    }
		LCD_SetTextColor(Yellow);
		LCD_DisplayStringLine(Line4, (uint8_t *) str2disp);
		// wyswietl bit parzystosci
		parity = parityBit(k);		// bit parzystosc
		LCD_SetTextColor(Red);
		LCD_DisplayChar(97, 100, parity ?'1':'0'); // wyswietla jeden znak
		
		Delay(8000000);
	}
	
	
	// petla glowna ---------------------------------------------
  while (1) {
			GPIO_WriteBit(GPIOF, GPIO_Pin_6, Bit_SET);
			Delay(4000000);
			GPIO_WriteBit(GPIOF, GPIO_Pin_6, Bit_RESET);
			GPIO_WriteBit(GPIOF, GPIO_Pin_7, Bit_SET);
			Delay(4000000);
			GPIO_WriteBit(GPIOF, GPIO_Pin_7, Bit_RESET);
			GPIO_WriteBit(GPIOF, GPIO_Pin_8, Bit_SET);
			Delay(4000000);
			GPIO_WriteBit(GPIOF, GPIO_Pin_8, Bit_RESET);
			GPIO_WriteBit(GPIOF, GPIO_Pin_9, Bit_SET);
			Delay(4000000);
			GPIO_WriteBit(GPIOF, GPIO_Pin_9, Bit_RESET);
			GPIO_WriteBit(GPIOF, GPIO_Pin_8, Bit_SET);
			Delay(4000000);
			GPIO_WriteBit(GPIOF, GPIO_Pin_8, Bit_RESET);
			GPIO_WriteBit(GPIOF, GPIO_Pin_7, Bit_SET);
			Delay(4000000);
			GPIO_WriteBit(GPIOF, GPIO_Pin_7, Bit_RESET);
  }
//  return 0;  // tego 'return' nigdy nie osiagnie 
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE);// wlacz taktowanie portu GPIO F
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  /*Tu nalezy umiescic kod zwiazany z konfiguracja poszczegolnych portow GPIO potrzebnych w programie*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOF, &GPIO_InitStructure);
	
}

void Delay(unsigned int counter){
	// opoznienie programowe, 8'000'000 to jedna sekunda
	while (counter--){
		__NOP();
		__NOP();
	}
}





