/******************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 	*
 *  STM3210E-EVAL: PROJEKT_24 Modbus KLIENT						*
 *			 CN12 (DB9M) - USART1 Master									*
 ******************************************************/
#include "stm32f10x.h"
#include "stm3210e_eval_lcd.h"
#include <stdio.h>

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
void DMA_Config(void);
void USART_Config(void);
int SysTick_Config_Div8(unsigned int Ticks);
void Delay(unsigned int);
uint16_t ModRTU_CRC(uint8_t bufor[], int len);

// zmienne globalne -------------------------------------------
unsigned char txt2LCD[32] = {0};// bufor funkcji sprintf
uint8_t Tx_message[32]={0};			// bufor na znaki do nadania
uint8_t Rx_message[32]={0};			// bufor na znaki odebrane
unsigned int Tx_num = 0;				// licznik ramek nadanych
unsigned int Rx_num = 0;				// licznik ramek odebranych
float current, T;

int main(void) {
  //------------------- deklaracje zmiennych ------------------
	unsigned int crc;
	char * wskaznik;
	
  //konfiguracja systemu --------------------------------------
  RCC_Config();		
  GPIO_Config();
	NVIC_Config();
	USART_Config();
	DMA_Config();
	USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE); 	// zezwolenie na DMA od USART1_Tx
	USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE); 	// zezwolenie na DMA od USART1_Rx
	STM3210E_LCD_Init();		// inicjalizacja TFT LCD 240320L8
  GPIO_ResetBits(GPIOF, GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9);	// zgasimy wszystkie diody LED
	LCD_Clear(Black);		// Clear the LCD
  LCD_SetBackColor(Black);		// kolor tla
	LCD_SetFont(&Font16x24);
	
	// przygotuje ramke do nadania: funkcja 03
	Tx_message[0] = 0x01;		// adres slave
	Tx_message[1] = 0x03;		// funkcja
	Tx_message[2] = 0x00;		// adres pierwszego rejestru
	Tx_message[3] = 0x00;
	Tx_message[4] = 0x00;		// ilosc rejestrow
	Tx_message[5] = 0x02;
	crc = ModRTU_CRC(Tx_message, 6);
	wskaznik = (char *) &crc;
	Tx_message[6] = wskaznik[0];
	Tx_message[7] = wskaznik[1];
	
	SysTick_Config_Div8(9000000ul);	// przerwanie co 1 sekunde
	
	txt2LCD[31] = 0x00;
	LCD_SetTextColor(Blue);   // kolor textu
	LCD_DisplayStringLine(Line0, (uint8_t *)"MASTER ramki:");
	
	while(1){
		//wyswietlimy liczniki transmisji
		LCD_SetTextColor(Green);   // kolor textu
		snprintf((char *)txt2LCD,31," nadane   = %i ",Tx_num);
		LCD_DisplayStringLine(Line1, (uint8_t *)txt2LCD);
		LCD_SetTextColor(Red);   // kolor textu
		snprintf((char *)txt2LCD,31," odebrane = %i ",Rx_num);
		LCD_DisplayStringLine(Line2, (uint8_t *)txt2LCD);
		// wyswietlimy osiem pierwszych nadanych znakow jako HEX
		snprintf((char *)txt2LCD,31,"Tx:%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X",Tx_message[0],Tx_message[1],Tx_message[2],Tx_message[3],Tx_message[4],Tx_message[5],Tx_message[6],Tx_message[7]);
		LCD_SetTextColor(Yellow);   // kolor textu
		LCD_DisplayStringLine(Line4, (uint8_t *)txt2LCD);
		// wyswietlimy osiem pierwszych odebranych znakow jako HEX
		snprintf((char *)txt2LCD,31,"Rx:%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X",Rx_message[0],Rx_message[1],Rx_message[2],Rx_message[3],Rx_message[4],Rx_message[5],Rx_message[6],Rx_message[7]);
		LCD_SetTextColor(White);   // kolor textu
		LCD_DisplayStringLine(Line5, (uint8_t *)txt2LCD);
		// wyswietlimy piec kolejnych odebranych znakow jako HEX
		snprintf((char *)txt2LCD,31,"   %2.2X%2.2X%2.2X%2.2X%2.2X",Rx_message[8],Rx_message[9],Rx_message[10],Rx_message[3],Rx_message[11]);
		LCD_DisplayStringLine(Line6, (uint8_t *)txt2LCD);
		
		// wyswietlimy odczytane liczby float
		LCD_SetTextColor(Green);   // kolor textu
		txt2LCD[31] = 0x00;
		snprintf((char *)txt2LCD,31,"  T = %3.2f  ['C]    ",T);
		LCD_DisplayStringLine(Line8, (uint8_t *)txt2LCD);
		LCD_SetTextColor(Blue);   // kolor textu
		snprintf((char *)txt2LCD,31,"CN3 = %3.3f [mA]    ",current);
		LCD_DisplayStringLine(Line9, (uint8_t *)txt2LCD);
	
		Delay(2000000);
	}
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii
		RCC_ADCCLKConfig(RCC_PCLK2_Div6);                    // ADCCLK = PCLK2/6 = 72 MHz /6 = 12 MHz
		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   // wlacz taktowanie DMA
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE);// port GPIO F : LEDy
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);// port GPIO A : PA9 (Tx), PA10 (Rx)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);// wlacz taktowanie USART1
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  /*Tu nalezy umiescic kod zwiazany z konfiguracja poszczegolnych portow GPIO potrzebnych w programie*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOF, &GPIO_InitStructure);
	
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;							// PA9 -> Tx
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
 	GPIO_Init(GPIOA, &GPIO_InitStructure);

 	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;						// PA10 <- Rx
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
 	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
 	GPIO_Init(GPIOA, &GPIO_InitStructure);
}
// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
  NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0); 
	
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
 	//Przerwanie od DMA1_Channel5_IRQn 
 	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel5_IRQn ;
 	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
 	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
 	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
 	NVIC_Init(&NVIC_InitStructure);
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}

// ----------------- konfiguracja kanalu DMA ------------------
void DMA_Config(void){
  #define USART1_DR_Address 0x40013804;                                       //adres rejestru USART1_DR
  DMA_InitTypeDef DMA_InitStructure;
	
  DMA_DeInit(DMA1_Channel4);                                                  // usun ewentualna poprzednia konfiguracje DMA
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)USART1_DR_Address;  		// adres USART
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) Tx_message;            		// adres pamieci
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;                          // kierunek transferu: pamiec -> USART
  DMA_InitStructure.DMA_BufferSize = 0x08;                                   	// liczba elementow do przeslania (dlugosc bufora)
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;            // wylaczenie autoinkrementacji adresu po stronie UART
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;                     // wlaczenie autoinkrementacji adresu po stronie pamieci (bufora)
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;			// rozmiar pojedynczych przesylanych danych po stronie ADC (Byte = 8bit)
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;         		// rozmiar pojedynczych przesylanych danych po stronie pamieci (bufora) = 8bit
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;                             	// tryb dzialania kontrolera DMA - jeden transfer
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;                         // priorytet DMA - wysoki
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                // wylaczenie obslugi transferu z pamieci do pamieci
  DMA_Init(DMA1_Channel4, &DMA_InitStructure);                                // zapis konfiguracji
//	DMA_Cmd(DMA1_Channel4, ENABLE);																							// wlacz DMA, kanal 4
	
	DMA_DeInit(DMA1_Channel5);                                                  // usun ewentualna poprzednia konfiguracje DMA
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)USART1_DR_Address;  		// adres USART
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) Rx_message;            		// adres pamieci
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;                          // kierunek transferu: USART -> pamiec
  DMA_InitStructure.DMA_BufferSize = 13;                       	            	// liczba elementow do przeslania (dlugosc bufora)
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;            // wylaczenie autoinkrementacji adresu po stronie UART
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;                     // wlaczenie autoinkrementacji adresu po stronie pamieci (bufora)
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;			// rozmiar pojedynczych przesylanych danych po stronie ADC (Byte = 8bit)
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;         		// rozmiar pojedynczych przesylanych danych po stronie pamieci (bufora) = 8bit
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;                             	// tryb dzialania kontrolera DMA - jeden transfer
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;                         // priorytet DMA - wysoki
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                // wylaczenie obslugi transferu z pamieci do pamieci
  DMA_Init(DMA1_Channel5, &DMA_InitStructure);                                // zapis konfiguracji
	DMA_Cmd(DMA1_Channel5, ENABLE);																							// wlacz DMA, kanal 5
	
	DMA_ITConfig(DMA1_Channel5, DMA_IT_TC, ENABLE);
}
// ------------------- konfiguracja USART1 --------------------
void USART_Config(void){
  USART_InitTypeDef  USART_InitStructure; 

  USART_InitStructure.USART_BaudRate = 9600;                                      // predkosc transmisji =9600bps
  USART_InitStructure.USART_WordLength = USART_WordLength_9b;                     // 8 bitow danych + PARITY
  USART_InitStructure.USART_StopBits = USART_StopBits_1;                          // 1 bit stopu
  USART_InitStructure.USART_Parity = USART_Parity_Even;                           // parzystosc
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; // bez kontroli przeplywu danych
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;	                // nadawanie i odbior
	USART_Init(USART1, &USART_InitStructure);

	USART_Cmd(USART1, ENABLE);
}
// ----------------- obliczamy CRC Modbus RTU -----------------
uint16_t ModRTU_CRC(uint8_t bufor[], int len){
  uint16_t crc = 0xFFFF;
	volatile unsigned int pos, i;

  for (pos = 0; pos < len; pos++) {
    crc ^= (uint16_t)bufor[pos];		// XOR kolejnego bajtu z CRC
 
    for (i = 8; i != 0; i--){				// petla po wszystkich osmiu bitach
      if ((crc & 0x0001) != 0){			// jezeli LSB jest jedynka, to ...
        crc >>= 1;                  // przesun w prawo i XOR 0xA001
        crc ^= 0xA001;
      }
      else                            // jesli LSB jest zerem, to ...
        crc >>= 1;                    // tylko przesun
    }
  }
  // UWGA: w Modbus RTU, CRC jest wysylane: mlodszy, starszy bajt
	return crc;  
}
// ------------------ opoznienie programowe -------------------
void Delay(unsigned int counter){
	// opoznienie programowe, 8'000'000 to jedna sekunda
	while (counter--){
		__NOP();
		__NOP();
	}
}
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/




